from .base_dgp import BaseDataGenusProcessor


class ConfigurableDGP(BaseDataGenusProcessor):

    def init(self, kind):
        self._flows = None
        self._analyzers = None
        self._kind = kind

    def analyze(self):
        if not self.steps:
            if self.context.taxonomy:
                if self.analyzers is not None:
                    self.steps = self.init_classes(self.analyzers)
        return super().analyze()

    @property
    def module(self):
        return getattr(self.context.taxonomy, self._kind)

    @property
    def flows(self):
        if self._flows is None:
            self._flows = self.module.flows(self.config, self.context)
        return self._flows

    @property
    def analyzers(self):
        if self._analyzers is None:
            self._analyzers = self.module.analyzers(self.config, self.context)
        return self._analyzers

    def preflow(self):
        if self.flows:
            return self.flows[0]

    def flow(self):
        from dataflows import Flow
        if self.flows:
            return Flow(
                self.flows[1],
            )
        elif self.analyzers:
            return super().flow()
        else:
            return Flow(
            )
